/*
 * 
 */
package org.openoffice.sdk.components;

import com.sun.star.awt.MessageBoxButtons;
import com.sun.star.awt.PosSize;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.registry.XRegistryKey;
import com.sun.star.lang.XSingleComponentFactory;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.lib.uno.helper.Factory;
import com.sun.star.lib.uno.helper.WeakBase;
import com.sun.star.frame.XFrame;
import com.sun.star.beans.XPropertySet;

import com.sun.star.awt.XDialog;
import com.sun.star.awt.XDialogProvider2;
import com.sun.star.awt.XControl;
import com.sun.star.awt.XControlModel;
import com.sun.star.awt.XControlContainer;
import com.sun.star.awt.XToolkit;
import com.sun.star.awt.XWindowPeer;
import com.sun.star.awt.XMessageBox;
import com.sun.star.awt.Rectangle;
import com.sun.star.awt.XMessageBoxFactory;
import com.sun.star.awt.XWindow;
import com.sun.star.uno.AnyConverter;


public final class DialogComponent
        extends     WeakBase
        implements  com.sun.star.lang.XServiceInfo,
                    com.sun.star.awt.XDialogEventHandler,
                    org.openoffice.sdk.components.XTestDialogHandler {

    private final XComponentContext m_xContext;
    private static final String m_implementationName = DialogComponent.class.getName();
    private static final String[] m_serviceNames = {
        "org.openoffice.sdk.components.TestDialogHandler"
    };
    private XFrame m_xFrame;
    private XWindow m_xContainerWindow;
    private XWindowPeer m_xDlgWindowPeer;
    
    private String m_aHandlerMethod1 = "doit1";
    private String m_aHandlerMethod2 = "doit2";
    private String m_aHandlerMethod3 = "doit3";

    public DialogComponent(XComponentContext context) {
        m_xContext = context;
    }
    
    //*************************************************************************

    public static XSingleComponentFactory __getComponentFactory(String sImplementationName) {
        XSingleComponentFactory xFactory = null;

        if (sImplementationName.equals(m_implementationName)) {
            xFactory = Factory.createComponentFactory(DialogComponent.class, m_serviceNames);
        }
        return xFactory;
    }

    public static boolean __writeRegistryServiceInfo(XRegistryKey xRegistryKey) {
        return Factory.writeRegistryServiceInfo(m_implementationName,
                m_serviceNames,
                xRegistryKey);
    }
    
    //*************************************************************************

    // com.sun.star.lang.XServiceInfo:
    
    public String getImplementationName() {
        return m_implementationName;
    }

    public boolean supportsService(String sService) {
        int len = m_serviceNames.length;

        for (int i = 0; i < len; i++) {
            if (sService.equals(m_serviceNames[i])) {
                return true;
            }
        }
        return false;
    }

    public String[] getSupportedServiceNames() {
        return m_serviceNames;
    }
    
    //*************************************************************************

    // com.sun.star.awt.XDialogEventHandler:
    
    public boolean callHandlerMethod(
                                        com.sun.star.awt.XDialog xDialog, 
                                        Object EventObject, 
                                        String MethodName) 
            throws com.sun.star.lang.WrappedTargetException 
    {
        if (MethodName.equals(m_aHandlerMethod1)) {
            showMessageBox(
                    m_xDlgWindowPeer,
                    "DialogComponent",
                    "callHandlerMethod() handled \"" + m_aHandlerMethod1 + "\"");
            return true;
        } else if (MethodName.equals(m_aHandlerMethod2)) {
            showMessageBox(
                    m_xDlgWindowPeer,
                    "DialogComponent",
                    "callHandlerMethod() handled \"" + m_aHandlerMethod2 + "\"");
            return true;
        } else if (MethodName.equals(m_aHandlerMethod3)) {
            showMessageBox(
                    m_xDlgWindowPeer,
                    "DialogComponent",
                    "callHandlerMethod() handled \"" + m_aHandlerMethod3 + "\"");
            return true;
        }
        return false;
    }

    public String[] getSupportedMethodNames() {
        String[] retValue = {
            m_aHandlerMethod1,
            m_aHandlerMethod2,
            m_aHandlerMethod3
        };
        return retValue;
    }

    //*************************************************************************
    
    // org.openoffice.sdk.devguide.components.XTestDialogHandler:
    
    public String createDialog(
                                String DialogURL, 
                                com.sun.star.frame.XModel xModel, 
                                com.sun.star.frame.XFrame xFrame) 
    {
        m_xFrame = xFrame;

        try {
            XMultiComponentFactory xMCF = m_xContext.getServiceManager();
            Object obj;

            // If valid we must pass the XModel when creating a DialogProvider object
            if (xModel != null) {
                Object[] args = new Object[1];
                args[0] = xModel;

                obj = xMCF.createInstanceWithArgumentsAndContext(
                        "com.sun.star.awt.DialogProvider2", args, m_xContext);
            } else {
                obj = xMCF.createInstanceWithContext(
                        "com.sun.star.awt.DialogProvider2", m_xContext);
            }

            XDialogProvider2 xDialogProvider = (XDialogProvider2) UnoRuntime.queryInterface(
                    XDialogProvider2.class, obj);

            XDialog xDialog = xDialogProvider.createDialogWithHandler(DialogURL, this);
            
            XWindow xDlgWindow = (XWindow) UnoRuntime.queryInterface(
                XWindow.class, xDialog);
            
            XControl xDlgControl = (XControl) UnoRuntime.queryInterface(
                XControl.class, xDialog);
            m_xDlgWindowPeer = xDlgControl.getPeer();
            
            m_xContainerWindow = m_xFrame.getContainerWindow();
            
            if (m_xContainerWindow == null){
                centerDialogOnScreen(xDlgWindow);
            } else {
                centerDialog(xDlgWindow, m_xContainerWindow.getPosSize());
            }
            

            if (xDialog != null) {
                xDialog.execute();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "Created dialog \"" + DialogURL + "\"";
    }

    public void copyText(
                            com.sun.star.awt.XDialog xDialog, 
                            Object aEventObject) 
    {
        XControlContainer xControlContainer = (XControlContainer) UnoRuntime.queryInterface(
                XControlContainer.class, xDialog);
        
        String aTextPropertyStr = "Text";
        String aText = "";
        
        XControl xTextField1Control = xControlContainer.getControl("TextField1");
        XControlModel xControlModel1 = xTextField1Control.getModel();
        
        XPropertySet xPropertySet1 = (XPropertySet) UnoRuntime.queryInterface(
                XPropertySet.class, xControlModel1);
        try {
            aText = AnyConverter.toString(
                    xPropertySet1.getPropertyValue(aTextPropertyStr));
        } catch (Exception e) {
            e.printStackTrace();
        }

        XControl xTextField2Control = xControlContainer.getControl("TextField2");
        XControlModel xControlModel2 = xTextField2Control.getModel();
        XPropertySet xPropertySet2 = (XPropertySet) UnoRuntime.queryInterface(
                XPropertySet.class, xControlModel2);
        try {
            xPropertySet2.setPropertyValue(aTextPropertyStr, aText);
        } catch (Exception e) {
            e.printStackTrace();
        }

        showMessageBox(
                    m_xDlgWindowPeer,"DialogComponent", "copyText() called");
    }

    public void handleEvent() {
        showMessageBox(
                    m_xDlgWindowPeer,"DialogComponent", "handleEvent() called");
    }

    public void handleEventWithArguments(
                                            com.sun.star.awt.XDialog xDialog, 
                                            Object aEventObject) 
    {
        showMessageBox(
                    m_xDlgWindowPeer,"DialogComponent", 
                    "handleEventWithArguments() called\n\n" +
                    "Event Object = " + aEventObject);
    }
    
    //************************************************************************

    public void showMessageBox(
                                XWindowPeer xWindowPeer, 
                                String sTitle, 
                                String sMessage) 
    {
        try {
            if (xWindowPeer!=null) {
                
                XToolkit xToolkit = xWindowPeer.getToolkit();
                
                XMessageBoxFactory xMessageBoxFactory = (XMessageBoxFactory)
                        UnoRuntime.queryInterface(XMessageBoxFactory.class, xToolkit);
                
                XMessageBox xMsgBox = xMessageBoxFactory.createMessageBox(
                        xWindowPeer, new Rectangle(), "infobox", 
                        MessageBoxButtons.BUTTONS_OK, sTitle, sMessage);
                xMsgBox.execute();
            }
        } catch (Exception ex) {
            // do your error handling 
            ex.printStackTrace();
        }
    }
    
    
    //*************************************************************************
    
    
    private void centerDialogOnScreen(XWindow xDialogWindow) {
        try {
            java.awt.Toolkit sysToolkit = java.awt.Toolkit.getDefaultToolkit();
            java.awt.Dimension screenSize = sysToolkit.getScreenSize();
            Rectangle screenPosSize = new Rectangle(0, 0, screenSize.width, screenSize.height);
            centerDialog(xDialogWindow, screenPosSize);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void centerDialog(
                                XWindow xDialogWindow, 
                                Rectangle parentWindowPosSize) 
    {
        try {
            Rectangle CurPosSize = xDialogWindow.getPosSize();

            int WindowHeight = parentWindowPosSize.Height;
            int WindowWidth = parentWindowPosSize.Width;

            int DialogWidth = CurPosSize.Width;
            int DialogHeight = CurPosSize.Height;

            int iXPos = ((WindowWidth / 2) - (DialogWidth / 2));
            int iYPos = ((WindowHeight / 2) - (DialogHeight / 2));

            xDialogWindow.setPosSize(
                    iXPos, iYPos,
                    DialogWidth, DialogHeight,
                    PosSize.POS);

        } catch (java.lang.Exception e) {
            e.printStackTrace();
        }
    }
    
}
